#library(Biobase)
#else library(GSEABase)

# UPDATED AUGUST 2008 TO THIN OUT

# snpLocs -- use resources in Herve's SNPlocs package (with
# an interim step, see vignette newSnpLoc.Rnw

setClass("snpLocs", representation(locEnv="environment",
  offsets="numeric", organism="character", versions="character"))

#setMethod("show", "snpLocs", function(object) {
# cat("snpLocs instance, organism ", object@organism, "\n")
# cat("based on:\n")
# print(object@versions)
#})

# casting: chrnum

setClass("chrnum", contains="character")
setMethod("show", "chrnum", function(object) {
 cat("GGtools chrnum instance:\n")
 callNextMethod()
})
setGeneric("chrnum", function(x)standardGeneric("chrnum"))
setMethod("chrnum", "character", function(x)new("chrnum", x))
setMethod("chrnum", "numeric", function(x)new("chrnum", as.character(x)))

# casting: rsid

setClass("rsid", contains="character")
setMethod("show", "rsid", function(object) {
 cat("GGtools rsid instance:\n")
 callNextMethod()
})
setGeneric("rsid", function(x)standardGeneric("rsid"))
setMethod("rsid", "character", function(x)new("rsid", x))
setMethod("rsid", "numeric", function(x)new("rsid", as.character(x)))

# GGtools infrastructure from hm2ceu, Mar 31 2008 (c) VJ Carey
# revised Aug 2008


# bump class version to reflect accommodation of new eSet protocolData slot
setClass("smlSet", contains="eSet", 
   representation(smlEnv="environment", annotation="character",
     organism="character"),
   prototype=prototype(
       new("VersionedBiobase",
               versions=c(classVersion("eSet"), smlSet="1.1.2")),
           phenoData = new("AnnotatedDataFrame",
             data=data.frame(),
             varMetadata=data.frame(
               labelDescription=character(0))),
	   annotation=character(0),
	   organism=character(0),
#	   smlEnv = {e = new.env(); assign("smList", list(), e); e},  # this does not satisfy validity check
	   smlEnv = {e = new.env(); nl = list(chr1=matrix(), chr2=matrix()); assign("smList", nl, e); e},
           chromInds = numeric(0)))

setGeneric("smlEnv", function(x) standardGeneric("smlEnv"))
setMethod("smlEnv", "smlSet", function(x) x@smlEnv)
setGeneric("smList", function(x) standardGeneric("smList"))
setMethod("smList", "smlSet", function(x) x@smlEnv$smList)

# drop in Jan 2011
setMethod("exprs", "smlSet", function(object) {
  object@assayData$exprs}
)

valsml = function(object) {
 nexsamp = ncol(exprs(object))
 allns = sapply(smList(object), nrow)
 if (!(all(allns==allns[1]))) 
    return("varying numbers of rows in elements of smList")
 if (!(all(allns==nexsamp)))
    return("some SnpMatrix instances have nrows != ncol(exprs(smlSet))")
# if ((sl <- length(smList(object))) != (cl <- length(object@chromInds)))
#    return(paste("length of chromInds vector [", cl, "] not identical to that of smList(object) [",
#      sl, "]"))
# nna = names(annotation(object))
# if ((length(nna) != 2) || (!(all(nna == c("exprs", "snps")))))
#    return("annotation slot must be vector with names 'exprs' and 'snps'")
 if (is.null(names(smList(object))))
     return("smList elements must bear names e.g., c(1:22,'X', 'Y')")
 return(TRUE)
}

setValidity("smlSet", valsml)

# more casting

setClass("genesym", contains="character")
setGeneric("genesym", function(x) standardGeneric("genesym"))
setMethod("genesym", "character",  function(x) new("genesym", x))

setClass("probeId", contains="character")
setGeneric("probeId", function(x) standardGeneric("probeId"))
setMethod("probeId", "character",  function(x) new("probeId", x))

setClass("snpdepth", contains="numeric")
snpdepth = function(x) new("snpdepth", x)

setClass("phenoVar", contains="character")
setGeneric("phenoVar", function(x)standardGeneric("phenoVar"))
setMethod("phenoVar", "character", function(x)new("phenoVar", as.character(x)))

setClass("smlSummary", contains="list")
setMethod("show", "smlSummary", function(object) {
 cat("smList summary of length", length(object), "\n")
 cat("excerpt:\n")
 print(object[[1]][1:3,])
 cat("\n")
})

setClass("multiCisTestResult", representation(conditions="list",
   call="call"), contains="list")
setMethod("show", "multiCisTestResult", function(object) {
 cat("GGBase multiCisTestResult container.\n")
 cat("Call was:\n")
 print(object@call)
 cat("There are ", length(object), "results.\n")
 cat("Conditions raised for", sum(sapply(object@conditions, function(x)
     !is.na(x$cond))), "genes.\n")
})

setMethod("updateObject", "smlSet", function(object, ..., verbose=FALSE) {
  make_smlSet(as(object, "ExpressionSet"), smList(object), ...)
})

#setClass("fsmlSet", contains="smlSet")
#
#setAs("smlSet", "fsmlSet", function(from, to) {
#  z = smList(from)
#  fns = paste(substitute(from), "_", names(z), ".ff", sep="")
#  ffreflist = lapply( 1:length(z), function(w)
#     ff(initdata=z[[w]]@.Data, dim=dim(z[[w]]), vmode="raw", overwrite=TRUE,
#          dimnames=NULL, filename=fns[w]))
#  names(ffreflist) = names(z)
#  smlEnv = {e = new.env(); assign("smList", ffreflist, e); e}
#  new(to, smlEnv=smlEnv, annotation=from@annotation, organism=from@organism,
#        assayData=from@assayData, phenoData=from@phenoData, featureData=from@featureData,
#        experimentData=from@experimentData, protocolData=from@protocolData)
#})

